#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 11-Jul-2003
################################################################################
#
# Audit - Auditing information for supervisory and administrative views
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

package eRealms::Audit;						# Package definition

use Socket;							# Socket library

################################################################################
# Sub-Routines
################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data members

	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $addr     = $cgi->remote_host();			# IP address
	my $pked;						# Packed binary address


	#####################
	# Program area

	# Invalid IP address
	if ($addr !~ /\d{1,3}\.\d{1,3}\.\d{1,3}/)
	{
		# Retrieve address
		$pked = gethostbyname($addr);

		# Form into human readable
		$addr = inet_ntoa($pked);
	}


	# Define self reference
	my $self = { 
		CGI	=> $cgi,
		DB	=> $database,
		HOST	=> $addr,
		USER	=> $param->{'USER'}
	 };


	# Bess to form class
	bless $self, $class;
	return $self;
}



#####################
# Audit Trail
#
# Used to create an audit trail of all supervisory and administrative changes.  
# This can be useful to see who is overriding a command or granting priviliges 
# without concent or knowledge.

sub AuditTrail							# Database, Parameters
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $trans    = shift;					# Transaction type
	my $action   = shift;					# Action taken
	my $code     = shift;					# Action code
	my $addition = shift;					# Additional code
	my $realm    = shift;					# Realm name

	my $database = $self->{'DB'};				# Database handle
        my $cgi      = $self->{'CGI'};				# CGI handle	


	#####################
	# Program area

	# Ensure definition
	$realm = 'system' unless (defined($realm));

	# Insert into audit trail
	if ($trans < 1)
	{
		# Insertion outside of transaction
		$database->Quick("INSERT INTO Audit 
			(PuppeteerLogin,
			 PuppeteerHostname,
			 AuditAction,
			 AuditCode,
			 AuditAdditional,
			 AuditRealm)
			VALUES (?,?,?,?,?,?)", 

		 $self->{'USER'}, 
		 $self->{'HOST'},
		 $action,
		 $code,
		 $addition,
		 $realm);
	}

	else 
	{
		# Transaction insertion
		$database->Write("INSERT INTO Audit 
			(PuppeteerLogin,
			 PuppeteerHostname,
			 AuditAction,
			 AuditCode,
			 AuditAdditional,
			 AuditRealm)
			VALUES (?,?,?,?,?,?)", 

		 $self->{'USER'}, 
		 $self->{'HOST'},
		 $action,
		 $code,
		 $addition,
		 $realm);
	}
}


#
# User Events
#

# Authenthication
# Alias for authenthication
sub Auth
{ AuditTrail(shift, 1, 'auth', 1000, @_); }

# Alias for IP authenthication failure
sub Fail 
{ AuditTrail(shift, 0, 'auth', 1001, @_); }

# Alias for password authenthication failure
sub Pass
{ AuditTrail(shift, 0, 'auth', 1003, @_); }

# Password reset
sub PasswordChange
{ AuditTrail(shift, 1, 'update', 1100, @_); }

# Email reset
sub EmailChange
{ AuditTrail(shift, 1, 'update', 1103, @_); }

# Realm create
sub UserRealm
{ AuditTrail(shift, 1, 'insert', 1200, @_); }


# Gallery
# Addition
sub GalleryAdd
{ AuditTrail(shift, 1, 'insert', 1104, @_); }

# Removal
sub GalleryRem
{ AuditTrail(shift, 1, 'delete', 1105, @_); }


# Puppet
# Addition
sub PuppetAdd
{ AuditTrail(shift, 1, 'insert', 1101, @_); }

# Removal
sub PuppetRem
{ AuditTrail(shift, 1, 'delete', 1102, @_); }


#
# System level events
#

# Word restriction
# Addition
sub AdmWordAdd
{ AuditTrail(shift, 1, 'insert', 9312, @_); }

# Remove
sub AdmWordRem
{ AuditTrail(shift, 1, 'delete', 9313, @_); }


# Supervisory lists
# Addition
sub AccessAdd
{ AuditTrail(shift, 1, 'insert', 9002, @_); }

# Removal
sub AccessRem
{ AuditTrail(shift, 1, 'delete', 9003, @_); }


# Auditing
# Access
sub AccessAudit
{ AuditTrail(shift, 0, 'action', 9314, @_); }


# Banning
# System ban
sub BanSystem
{ AuditTrail(shift, 0, 'insert', 9000, @_); }


# Contact
# Addition
sub ContactAdd
{ AuditTrail(shift, 1, 'insert', 9200, @_); }

# Removal
sub ContactRem
{ AuditTrail(shift, 1, 'delete', 9201, @_); }


# Cross-Reference
sub CrossRef
{ AuditTrail(shift, 1, 'action', 9004, @_); }


# Document and Templates
# Update
sub Document
{ AuditTrail(shift, 1, 'update', 9300, @_); }


# Fonts
# Addition
sub FontAdd
{ AuditTrail(shift, 1, 'insert', 9100, @_); }

# Removal
sub FontRem
{ AuditTrail(shift, 1, 'delete', 9101, @_); }


# Gummies
# Addition
sub GummyAdd
{ AuditTrail(shift, 1, 'insert', 9202, @_); }

# Removal
sub GummyRem
{ AuditTrail(shift, 1, 'delete', 9203, @_); }


# Genre
# Addition
sub GenreAdd
{ AuditTrail(shift, 1, 'insert', 9305, @_); }

# Removal
sub GenreRem
{ AuditTrail(shift, 1, 'delete', 9306, @_); }

# Update
sub GenreUpdate
{ AuditTrail(shift, 1, 'update', 9307, @_); }


# Homepage
sub HomepageSpan
{ AuditTrail(shift, 1, 'update', 9005, @_); }

# Poll
# Addition
sub PollAdd
{ AuditTrail(shift, 1, 'insert', 9308, @_); }

# Removal
sub PollRem
{ AuditTrail(shift, 1, 'delete', 9309, @_); }


# Puppeteer
# Change
sub PuppeteerChange
{ AuditTrail(shift, 0, 'update', 9001, @_); }


# Poll
# Addition
sub QuestionAdd
{ AuditTrail(shift, 1, 'insert', 9310, @_); }

# Removal
sub QuestionRem
{ AuditTrail(shift, 1, 'delete', 9311, @_); }


# Realms
# Addition
sub RealmAdd
{ AuditTrail(shift, 1, 'insert', 9104, @_); }

# Removal
sub RealmRem
{ AuditTrail(shift, 1, 'delete', 9105, @_); }

# Public status
sub RealmStatus
{ AuditTrail(shift, 1, 'update', 9106, @_); }


# Supervisors
sub SuperStrip
{ AuditTrail(shift, 1, 'delete', 9107, @_); }

# Tags
# Change
sub TagChange
{ AuditTrail(shift, 1, 'update', 9302, @_); }


# User
# Removal
sub UserRem
{ AuditTrail(shift, 1, 'delete', 9006, @_); }


# Weblog handler
# Addition
sub WeblogAdd
{ AuditTrail(shift, 1, 'insert', 9303, @_); }

# Removal
sub WeblogRem
{ AuditTrail(shift, 1, 'delete', 9304, @_); }


#
# Realm events
#

# Alias for bannings
# Add
sub BanAdd
{ AuditTrail(shift, 1, 'insert', 2001, @_); }

# Remove
sub BanRem
{ AuditTrail(shift, 1, 'delete', 2002, @_); }


# Message Board
# Remove
sub BoardRem
{ AuditTrail(shift, 0, 'delete', 2100, @_); }


# Canon handles
# Addition
sub CanonAdd
{ AuditTrail(shift, 1, 'insert', 2003, @_); }

# Removal
sub CanonDel
{ AuditTrail(shift, 1, 'delete', 2004, @_); }


# Font
# Update
sub FontAdjust
{ AuditTrail(shift, 1, 'update', 2210, @_); }


# Forcast
# Update
sub Forcast
{ AuditTrail(shift, 1, 'update', 2209, @_); }


# Realm details
# Alias for Information
sub InfoChange
{ AuditTrail(shift, 1, 'update', 2203, @_); }

# Info reset
sub InfoReset
{ AuditTrail(shift, 1, 'update', 2204, @_); }


# Locale
# Addition
sub LocaleAdd
{ AuditTrail(shift, 1, 'insert', 2201, @_); }

# Removal
sub LocaleRem
{ AuditTrail(shift, 1, 'delete', 2202, @_); }


# Alias for Mods
sub ModsAdd
{ AuditTrail(shift, 1, 'insert', 2004, @_); }

sub ModsRem
{ AuditTrail(shift, 1, 'delete', 2005, @_); }


# Picture
# Alias for Pictures
sub PicAdd
{ AuditTrail(shift, 1, 'insert', 2207, @_); }

sub PicRem
{ AuditTrail(shift, 1, 'delete', 2208, @_); }


# Realm
# Update
sub RealmDetails
{ AuditTrail(shift, 1, 'update', 2207, @_); }


# Board Topics
# Addition
sub TopicAdd
{ AuditTrail(shift, 1, 'insert', 2101, @_); }

# Removal
sub TopicRem
{ AuditTrail(shift, 1, 'delete', 2102, @_); }

# Modification
sub TopicEdit
{ AuditTrail(shift, 1, 'update', 2103, @_); }


# Warnings
# Action
sub Warn
{ AuditTrail(shift, 1, 'action', 2007, @_); }


# Word restriction
# Addition
sub WordAdd
{ AuditTrail(shift, 1, 'insert', 2205, @_); }

# Remove
sub WordRem
{ AuditTrail(shift, 1, 'delete', 2206, @_); }

1;
